@@ -31,6 +31,7 @@ Sample use::
 from pathlib import Path
 import tempfile
 import numpy as np
+import os
 import sys
 import struct
 import gzip
@@ -49,43 +50,50 @@ class CDF:
     ----------
     path : Path, str
         Path to CDF file.
-    validate : bool
+    validate : bool, optional
         If True, validate the MD5 checksum of the CDF file.
+    string_encoding : str, optional
+        The encoding used to read strings. Defaults to 'ascii', which is what
+        the CDF internal format description prescribes as the encoding for
+        character strings. Other encodings may have been used to create files
+        however, and this keyword argument gives users the flexibility to read
+        those files.
     """
-
     version = 3
     release = 7
     increment = 0
 
-    def __init__(self, path, validate=False):
+    def __init__(self, path, validate=False, string_encoding='ascii'):
 
         path = Path(path).expanduser()
         if not path.is_file():
             path = path.with_suffix('.cdf')
             if not path.is_file():
-                raise FileNotFoundError('{} not found'.format(path))
+                raise FileNotFoundError(f'{path} not found')
 
         self.file = path
+        self.string_encoding = string_encoding
 
         with self.file.open('rb') as f:
             magic_number = f.read(4).hex()
             compressed_bool = f.read(4).hex()
 
         if magic_number not in ('cdf30001', 'cdf26002', '0000ffff'):
-            raise OSError('{} is not a CDF file or a non-supported CDF!'.format(path))
+            raise OSError(f'{path} is not a CDF file or a non-supported CDF!')
 
         self.cdfversion = 3 if magic_number == 'cdf30001' else 2
 
         self._compressed = not (compressed_bool == '0000ffff')
         self.compressed_file = None
+        self.temp_file = None
 
         if self._compressed:
-            new_path = self._uncompress_file(path)
-            if new_path is None:
+            self._uncompress_file(path)
+            if self.temp_file is None:
                 raise OSError("Decompression was unsuccessful.  Only GZIP compression is currently supported.")
 
             self.compressed_file = self.file
-            self.file = new_path
+            self.file = self.temp_file
 
         if (self.cdfversion == 3):
             cdr_info, foffs = self._read_cdr(8)
@@ -129,9 +137,21 @@ class CDF:
         if self.compressed_file is not None:
             self.compressed_file = None
 
+    def __del__(self):
+        # This implicitly will delete a temporary uncompressed file if we
+        # created it earlier.
+        if self.temp_file is not None:
+            os.remove(self.temp_file)
+
     def __getitem__(self, variable: str) -> np.ndarray:
         return self.varget(variable)
 
+    def __enter__(self):
+        return self
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        return
+
     def close(self):
         pass
 
@@ -150,11 +170,11 @@ class CDF:
                 +---------------+--------------------------------------------------------------------------------+
                 | ['Majority']  | the row/column majority                                                        |
                 +---------------+--------------------------------------------------------------------------------+
-                | ['zVariables']| the dictionary for zVariable numbers and their corresponding names             |
+                | ['zVariables']| a list of the names of the zVariables                                          |
                 +---------------+--------------------------------------------------------------------------------+
-                | ['rVariables']| the dictionary for rVariable numbers and their corresponding names             |
+                | ['rVariables']| a list of the names of the rVariables                                          |
                 +---------------+--------------------------------------------------------------------------------+
-                | ['Attributes']| the dictionary for attribute numbers and their corresponding names and scopes  |
+                | ['Attributes']| a list of dictionary objects that contain {attribute_name : scope}             |
                 +---------------+--------------------------------------------------------------------------------+
                 | ['Checksum']  | the checksum indicator                                                         |
                 +---------------+--------------------------------------------------------------------------------+
@@ -225,7 +245,7 @@ class CDF:
         """
         vdr_info = self.varget(variable=variable, inq=True)
         if vdr_info is None:
-            raise KeyError("Variable {} not found.".format(variable))
+            raise KeyError(f"Variable {variable} not found.")
 
         var = {}
         var['Variable'] = vdr_info['name']
@@ -248,12 +268,6 @@ class CDF:
 
         return var
 
-    def _read_adr(self, position):
-        if (self.cdfversion == 3):
-            return self._read_adr3(position)
-        else:
-            return self._read_adr2(position)
-
     def attinq(self, attribute=None):
         """
         Get attribute information.
@@ -276,11 +290,11 @@ class CDF:
                     return self._read_adr(position)
 
                 position = next_adr
-            raise KeyError('No attribute {}'.format(attribute))
+            raise KeyError(f'No attribute {attribute}')
 
         elif isinstance(attribute, int):
             if (attribute < 0 or attribute > self._num_zvariable):
-                raise KeyError('No attribute {}'.format(attribute))
+                raise KeyError(f'No attribute {attribute}')
             for _ in range(0, attribute):
                 name, next_adr = self._read_adr_fast(position)
                 position = next_adr
@@ -346,13 +360,13 @@ class CDF:
                     position = next_adr
 
             if adr_info is None:
-                raise KeyError('No attribute {}'.format(attribute))
+                raise KeyError(f'No attribute {attribute}')
 
         elif isinstance(attribute, int):
             if (attribute < 0) or (attribute > self._num_att):
-                raise KeyError('No attribute {}'.format(attribute))
+                raise KeyError(f'No attribute {attribute}')
             if not isinstance(entry, int):
-                raise TypeError('{} has to be a number.'.format(entry))
+                raise TypeError(f'{entry} has to be a number.')
 
             for _ in range(0, attribute):
                 name, next_adr = self._read_adr_fast(position)
@@ -377,10 +391,7 @@ class CDF:
                 # a zVariable?
                 positionx = self._first_zvariable
                 for x in range(0, self._num_zvariable):
-                    if (self.cdfversion == 3):
-                        name, vdr_next = self._read_vdr_fast(positionx)
-                    else:
-                        name, vdr_next = self._read_vdr_fast2(positionx)
+                    name, vdr_next = self._read_vdr_fast(positionx)
                     if (name.strip().lower() == entry.strip().lower()):
                         var_num = x
                         zvar = True
@@ -390,17 +401,14 @@ class CDF:
                     # a rVariable?
                     positionx = self._first_rvariable
                     for x in range(0, self._num_rvariable):
-                        if (self.cdfversion == 3):
-                            name, vdr_next = self._read_vdr_fast(positionx)
-                        else:
-                            name, vdr_next = self._read_vdr_fast2(positionx)
+                        name, vdr_next = self._read_vdr_fast(positionx)
                         if (name.strip().lower() == entry.strip().lower()):
                             var_num = x
                             break
                         positionx = vdr_next
                 if var_num == -1:
                     raise ValueError(
-                        'No variable by this name: {}'.format(entry))
+                        f'No variable by this name: {entry}')
                 entry_num = var_num
             else:
                 if (self._num_zvariable > 0 and self._num_rvariable > 0):
@@ -429,6 +437,18 @@ class CDF:
         """
         Returns the variable data.
 
+        Parameters
+        ----------
+        variable: str
+            Variable name to fetch.
+        startrec: int
+            Index of the first record to get.
+        endrec : int
+            Index of the last record to get. All records from *startrec* to
+            *endrec* inclusive are fetched.
+
+        Notes
+        -----
         Variable can be entered either
         a name or a variable number. By default, it returns a
         'numpy.ndarray' or 'list' class object, depending on the
@@ -493,15 +513,9 @@ class CDF:
             vdr_info = None
             for zVar in [1, 0]:
                 for _ in range(0, num_variables):
-                    if (self.cdfversion == 3):
-                        name, vdr_next = self._read_vdr_fast(position)
-                    else:
-                        name, vdr_next = self._read_vdr_fast2(position)
+                    name, vdr_next = self._read_vdr_fast(position)
                     if name.strip().lower() == variable.strip().lower():
-                        if (self.cdfversion == 3):
-                            vdr_info = self._read_vdr(position)
-                        else:
-                            vdr_info = self._read_vdr2(position)
+                        vdr_info = self._read_vdr(position)
                         break
                     position = vdr_next
                 position = self._first_rvariable
@@ -519,17 +533,11 @@ class CDF:
                 # zVar = False
             if (variable < 0 or variable >= num_variable):
                 raise ValueError(
-                    'No variable by this number: {}'.format(variable))
+                    f'No variable by this number: {variable}')
             for _ in range(0, variable):
-                if (self.cdfversion == 3):
-                    name, next_vdr = self._read_vdr_fast(position)
-                else:
-                    name, next_vdr = self._read_vdr_fast2(position)
+                name, next_vdr = self._read_vdr_fast(position)
                 position = next_vdr
-            if (self.cdfversion == 3):
-                vdr_info = self._read_vdr(position)
-            else:
-                vdr_info = self._read_vdr2(position)
+            vdr_info = self._read_vdr(position)
         else:
             raise ValueError('Please set variable keyword equal to '
                              'the name or number of an variable')
@@ -538,7 +546,7 @@ class CDF:
             return vdr_info
         else:
             if (vdr_info['max_records'] < 0):
-                return
+                raise ValueError(f'No records found for variable {variable}')
 
             return self._read_vardata(vdr_info, epoch=epoch, starttime=starttime, endtime=endtime,
                                       startrec=startrec, endrec=endrec, record_range_only=record_range_only,
@@ -599,10 +607,7 @@ class CDF:
                 entries = {}
             aedr_byte_loc = adr_info['first_gr_entry']
             for _ in range(adr_info['num_gr_entry']):
-                if (self.cdfversion == 3):
-                    aedr_info = self._read_aedr(aedr_byte_loc, to_np=to_np)
-                else:
-                    aedr_info = self._read_aedr2(aedr_byte_loc, to_np=to_np)
+                aedr_info = self._read_aedr(aedr_byte_loc, to_np=to_np)
                 entryData = aedr_info['entry']
                 if (expand is False):
                     entries.append(entryData)
@@ -674,20 +679,14 @@ class CDF:
             num_variables = self._num_zvariable
             for zVar in [1, 0]:
                 for _ in range(0, num_variables):
-                    if (self.cdfversion == 3):
-                        name, vdr_next = self._read_vdr_fast(position)
-                    else:
-                        name, vdr_next = self._read_vdr_fast2(position)
+                    name, vdr_next = self._read_vdr_fast(position)
                     if name.strip().lower() == variable.strip().lower():
-                        if (self.cdfversion == 3):
-                            vdr_info = self._read_vdr(position)
-                        else:
-                            vdr_info = self._read_vdr2(position)
+                        vdr_info = self._read_vdr(position)
                         return self._read_varatts(vdr_info['variable_number'], zVar, expand, to_np=to_np)
                     position = vdr_next
                 position = self._first_rvariable
                 num_variables = self._num_rvariable
-            raise ValueError('No variable by this name: {}'.format(variable))
+            raise ValueError(f'No variable by this name: {variable}')
         elif isinstance(variable, int):
             if self._num_zvariable > 0:
                 num_variable = self._num_zvariable
@@ -696,7 +695,7 @@ class CDF:
                 num_variable = self._num_rvariable
                 zVar = False
             if (variable < 0 or variable >= num_variable):
-                raise ValueError('No variable by this number: {}'.format(variable))
+                raise ValueError(f'No variable by this number: {variable}')
             return self._read_varatts(variable, zVar, expand, to_np=to_np)
         else:
             raise ValueError('Please set variable keyword equal to '
@@ -720,14 +719,12 @@ class CDF:
             f.seek(data_start)
             decompressed_data = gzip.decompress(f.read(data_size))
 
-        newpath = Path(tempfile.NamedTemporaryFile(suffix='.cdf').name)
-        with newpath.open('wb') as g:
+        self.temp_file = Path(tempfile.NamedTemporaryFile(suffix='.cdf').name)
+        with self.temp_file.open('wb') as g:
             g.write(bytearray.fromhex('cdf30001'))
             g.write(bytearray.fromhex('0000ffff'))
             g.write(decompressed_data)
 
-        return newpath
-
     def _read_ccr(self, byte_loc):
         with self.file.open('rb') as f:
             f.seek(byte_loc, 0)
@@ -755,6 +752,12 @@ class CDF:
         return data_start, data_size, cType, cParams
 
     def _read_cpr(self, byte_loc):
+        if self.cdfversion == 3:
+            return self._read_cpr3(byte_loc)
+        else:
+            return self._read_cpr2(byte_loc)
+
+    def _read_cpr3(self, byte_loc):
         with self.file.open('rb') as f:
             f.seek(byte_loc, 0)
             block_size = int.from_bytes(f.read(8), 'big')
@@ -868,20 +871,14 @@ class CDF:
             position = self._first_zvariable
             num_variable = self._num_zvariable
             for _ in range(0, num_variable):
-                if (self.cdfversion == 3):
-                    name, next_vdr = self._read_vdr_fast(position)
-                else:
-                    name, next_vdr = self._read_vdr_fast2(position)
+                name, next_vdr = self._read_vdr_fast(position)
                 zvars.append(name)
                 position = next_vdr
         if self._num_rvariable > 0:
             position = self._first_rvariable
             num_variable = self._num_rvariable
             for _ in range(0, num_variable):
-                if (self.cdfversion == 3):
-                    name, next_vdr = self._read_vdr_fast(position)
-                else:
-                    name, next_vdr = self._read_vdr_fast2(position)
+                name, next_vdr = self._read_vdr_fast(position)
                 rvars.append(name)
                 position = next_vdr
         return rvars, zvars
@@ -898,6 +895,9 @@ class CDF:
         return attrs
 
     def _read_cdr(self, byte_loc: int):
+        """
+        Read a CDF descriptor record (CDR).
+        """
         with self.file.open('rb') as f:
             f.seek(byte_loc, 0)
             block_size = int.from_bytes(f.read(8), 'big')
@@ -907,26 +907,31 @@ class CDF:
         # gdroff = int.from_bytes(cdr[4:12], 'big')  # GDR Location
         version = int.from_bytes(cdr[12:16], 'big')
         if version not in (2, 3):
-            raise ValueError('CDF version {} not handled'.format(version))
+            raise ValueError(f'CDF version {version} not handled')
 
         release = int.from_bytes(cdr[16:20], 'big')
         encoding = int.from_bytes(cdr[20:24], 'big')
 
         # FLAG
         #
-        # 0 The majority of variable values within a variable record. Variable records are described in Chapter 4. Set indicates row-majority. Clear indicates column-majority.
-        # 1 The file format of the CDF. Set indicates single-file. Clear indicates multi-file.
+        # 0 The majority of variable values within a variable record.
+        #   Variable records are described in Chapter 4. Set indicates
+        #   row-majority. Clear indicates column-majority.
+        # 1 The file format of the CDF. Set indicates single-file.
+        #   Clear indicates multi-file.
         # 2 The checksum of the CDF. Set indicates a checksum method is used.
-        # 3 The MD5 checksum method indicator. Set indicates MD5 method is used for the checksum. Bit 2 must be set.
-        # 4 Reserved for another checksum method. Bit 2 must be set and bit 3 must be clear.
+        # 3 The MD5 checksum method indicator.
+        #   Set indicates MD5 method is used for the checksum. Bit 2 must be set.
+        # 4 Reserved for another checksum method.
+        #   Bit 2 must be set and bit 3 must be clear.
 
         flag = int.from_bytes(cdr[24:28], 'big')
-        flag_bits = '{0:032b}'.format(flag)
+        flag_bits = f'{flag:032b}'
         row_majority = (flag_bits[31] == '1')
         single_format = (flag_bits[30] == '1')
         md5 = (flag_bits[29] == '1' and flag_bits[28] == '1')
         increment = int.from_bytes(cdr[36:40], 'big')
-        cdfcopyright = cdr[48:].decode('utf-8')
+        cdfcopyright = cdr[48:].decode(self.string_encoding)
         cdfcopyright = cdfcopyright.replace('\x00', '')
 
         cdr_info = {}
@@ -956,12 +961,12 @@ class CDF:
         release = int.from_bytes(cdr[12:16], 'big')
         encoding = int.from_bytes(cdr[16:20], 'big')
         flag = int.from_bytes(cdr[20:24], 'big')
-        flag_bits = '{0:032b}'.format(flag)
+        flag_bits = f'{flag:032b}'
         row_majority = (flag_bits[31] == '1')
         single_format = (flag_bits[30] == '1')
         md5 = (flag_bits[29] == '1' and flag_bits[28] == '1')
         increment = int.from_bytes(cdr[32:36], 'big')
-        cdfcopyright = cdr[44:].decode('utf-8')
+        cdfcopyright = cdr[44:].decode(self.string_encoding)
         cdfcopyright = cdfcopyright.replace('\x00', '')
 
         cdr_info = {}
@@ -1069,19 +1074,13 @@ class CDF:
                 num_entry = adr_info['num_gr_entry']
             found = 0
             for _ in range(0, num_entry):
-                if (self.cdfversion == 3):
-                    entryNum, byte_next = self._read_aedr_fast(byte_loc)
-                else:
-                    entryNum, byte_next = self._read_aedr_fast2(byte_loc)
+                entryNum, byte_next = self._read_aedr_fast(byte_loc)
                 if (entryNum != var_num):
                     byte_loc = byte_next
                     continue
-                if (self.cdfversion == 3):
-                    aedr_info = self._read_aedr(byte_loc, to_np=to_np)
-                else:
-                    aedr_info = self._read_aedr2(byte_loc, to_np=to_np)
+                aedr_info = self._read_aedr(byte_loc, to_np=to_np)
                 entryData = aedr_info['entry']
-                if (expand == False):
+                if (expand is False):
                     return_dict[adr_info['name']] = entryData
                 else:
                     entryWithType = []
@@ -1112,15 +1111,24 @@ class CDF:
                 found = 1
                 break
             byte_loc = adr_info['next_adr_location']
-            if (found == 0 and expand != False):
+            if (found == 0 and expand is not False):
                 return_dict[adr_info['name']] = None
         return return_dict
 
+    def _read_adr(self, position):
+        """
+        Read an attribute descriptor record (ADR).
+        """
+        if (self.cdfversion == 3):
+            return self._read_adr3(position)
+        else:
+            return self._read_adr2(position)
+
     def _read_adr3(self, byte_loc):
         with self.file.open('rb') as f:
             f.seek(byte_loc, 0)
             block_size = int.from_bytes(f.read(8), 'big')  # Block Size
-            adr = f.read(block_size-8)
+            adr = f.read(block_size - 8)
         next_adr_loc = int.from_bytes(adr[4:12], 'big', signed=True)
         position_next_gr_entry = int.from_bytes(adr[12:20], 'big', signed=True)
         scope = int.from_bytes(adr[20:24], 'big', signed=True)
@@ -1131,7 +1139,7 @@ class CDF:
         num_z_entry = int.from_bytes(adr[48:52], 'big', signed=True)
         MaxZEntry = int.from_bytes(adr[52:56], 'big', signed=True)
 
-        name = str(adr[60:315].decode('utf-8'))
+        name = str(adr[60:315].decode(self.string_encoding))
         name = name.replace('\x00', '')
 
         # Build the return dictionary
@@ -1153,7 +1161,7 @@ class CDF:
         with self.file.open('rb') as f:
             f.seek(byte_loc, 0)
             block_size = int.from_bytes(f.read(4), 'big')  # Block Size
-            adr = f.read(block_size-4)
+            adr = f.read(block_size - 4)
 
         next_adr_loc = int.from_bytes(adr[4:8], 'big', signed=True)
         position_next_gr_entry = int.from_bytes(adr[8:12], 'big', signed=True)
@@ -1165,7 +1173,7 @@ class CDF:
         num_z_entry = int.from_bytes(adr[36:40], 'big', signed=True)
         MaxZEntry = int.from_bytes(adr[40:44], 'big', signed=True)
 
-        name = str(adr[48:112].decode('utf-8'))
+        name = str(adr[48:112].decode(self.string_encoding))
         name = name.replace('\x00', '')
 
         # Build the return dictionary
@@ -1184,6 +1192,9 @@ class CDF:
         return return_dict
 
     def _read_adr_fast(self, position):
+        """
+        Read an attribute descriptor record (ADR).
+        """
         if (self.cdfversion == 3):
             return self._read_adr_fast3(position)
         else:
@@ -1192,11 +1203,11 @@ class CDF:
     def _read_adr_fast3(self, byte_loc):
         with self.file.open('rb') as f:
             # Position of next ADR
-            f.seek(byte_loc+12, 0)
+            f.seek(byte_loc + 12, 0)
             next_adr_loc = int.from_bytes(f.read(8), 'big', signed=True)
             # Name
-            f.seek(byte_loc+68, 0)
-            name = str(f.read(256).decode('utf-8'))
+            f.seek(byte_loc + 68, 0)
+            name = str(f.read(256).decode(self.string_encoding))
 
         name = name.replace('\x00', '')
 
@@ -1205,30 +1216,36 @@ class CDF:
     def _read_adr_fast2(self, byte_loc):
         with self.file.open('rb') as f:
             # Position of next ADR
-            f.seek(byte_loc+8, 0)
+            f.seek(byte_loc + 8, 0)
             next_adr_loc = int.from_bytes(f.read(4), 'big', signed=True)
             # Name
-            f.seek(byte_loc+52, 0)
-            name = str(f.read(64).decode('utf-8'))
+            f.seek(byte_loc + 52, 0)
+            name = str(f.read(64).decode(self.string_encoding))
 
         name = name.replace('\x00', '')
 
         return name, next_adr_loc
 
     def _read_aedr_fast(self, byte_loc):
+        if self.cdfversion == 3:
+            return self._read_aedr_fast3(byte_loc)
+        else:
+            return self._read_aedr_fast2(byte_loc)
+
+    def _read_aedr_fast3(self, byte_loc):
         with self.file.open('rb') as f:
-            f.seek(byte_loc+12, 0)
+            f.seek(byte_loc + 12, 0)
             next_aedr = int.from_bytes(f.read(8), 'big', signed=True)
 
             # Variable number or global entry number
-            f.seek(byte_loc+28, 0)
+            f.seek(byte_loc + 28, 0)
             entry_num = int.from_bytes(f.read(4), 'big', signed=True)
 
         return entry_num, next_aedr
 
     def _read_aedr_fast2(self, byte_loc):
         with self.file.open('rb') as f:
-            f.seek(byte_loc+8, 0)
+            f.seek(byte_loc + 8, 0)
             next_aedr = int.from_bytes(f.read(4), 'big', signed=True)
 
             # Variable number or global entry number
@@ -1238,6 +1255,12 @@ class CDF:
         return entry_num, next_aedr
 
     def _read_aedr(self, byte_loc, to_np=True):
+        if self.cdfversion == 3:
+            return self._read_aedr3(byte_loc, to_np)
+        else:
+            return self._read_aedr2(byte_loc, to_np)
+
+    def _read_aedr3(self, byte_loc, to_np=True):
         '''
         Reads an Attribute Entry Descriptor Record at a specific byte location.
 
@@ -1313,16 +1336,25 @@ class CDF:
         return_dict['entry'] = entry
         return_dict['data_type'] = data_type
         return_dict['num_elements'] = num_elements
-        #return_dict['num_strings'] = num_strings
+        # return_dict['num_strings'] = num_strings
         return_dict['next_aedr'] = next_aedr
         return_dict['entry_num'] = entry_num
         return return_dict
 
     def _read_vdr(self, byte_loc):
+        """
+        Read a variable descriptor record (VDR).
+        """
+        if self.cdfversion == 3:
+            return self._read_vdr3(byte_loc)
+        else:
+            return self._read_vdr2(byte_loc)
+
+    def _read_vdr3(self, byte_loc):
         with self.file.open('rb') as f:
             f.seek(byte_loc, 0)
             block_size = int.from_bytes(f.read(8), 'big')
-            vdr = f.read(block_size-8)
+            vdr = f.read(block_size - 8)
         # Type of internal record
         section_type = int.from_bytes(vdr[0:4], 'big')
         next_vdr = int.from_bytes(vdr[4:12], 'big', signed=True)
@@ -1332,7 +1364,7 @@ class CDF:
         last_vxr = int.from_bytes(vdr[28:36], 'big', signed=True)
         flags = int.from_bytes(vdr[36:40], 'big', signed=True)
 
-        flag_bits = '{0:032b}'.format(flags)
+        flag_bits = f'{flags:032b}'
 
         record_variance_bool = (flag_bits[31] == '1')
         pad_bool = (flag_bits[30] == '1')
@@ -1343,7 +1375,7 @@ class CDF:
         var_num = int.from_bytes(vdr[60:64], 'big', signed=True)
         CPRorSPRoffset = int.from_bytes(vdr[64:72], 'big', signed=True)
         blocking_factor = int.from_bytes(vdr[72:76], 'big', signed=True)
-        name = str(vdr[76:332].decode('utf-8'))
+        name = str(vdr[76:332].decode(self.string_encoding))
         name = name.replace('\x00', '')
 
         zdim_sizes = []
@@ -1404,10 +1436,7 @@ class CDF:
             return_dict['pad'] = pad
         return_dict['compression_bool'] = compression_bool
         if (compression_bool):
-            if (self.cdfversion == 3):
-                ctype, cparm = self._read_cpr(CPRorSPRoffset)
-            else:
-                ctype, cparm = self._read_cpr2(CPRorSPRoffset)
+            ctype, cparm = self._read_cpr(CPRorSPRoffset)
             return_dict['compression_level'] = cparm
         else:
             return_dict['compression_level'] = 0
@@ -1420,7 +1449,7 @@ class CDF:
 
     def _read_vdr2(self, byte_loc):
 
-        if (self._post25 == True):
+        if (self._post25 is True):
             toadd = 0
         else:
             toadd = 128
@@ -1436,7 +1465,7 @@ class CDF:
         head_vxr = int.from_bytes(vdr[16:20], 'big', signed=True)
         last_vxr = int.from_bytes(vdr[20:24], 'big', signed=True)
         flags = int.from_bytes(vdr[24:28], 'big', signed=True)
-        flag_bits = '{0:032b}'.format(flags)
+        flag_bits = f'{flags:032b}'
         record_variance_bool = (flag_bits[31] == '1')
         pad_bool = (flag_bits[30] == '1')
         compression_bool = (flag_bits[29] == '1')
@@ -1448,7 +1477,7 @@ class CDF:
                                         signed=True)
         blocking_factor = int.from_bytes(vdr[56+toadd:60+toadd], 'big',
                                          signed=True)
-        name = str(vdr[60+toadd:124+toadd].decode('utf-8'))
+        name = str(vdr[60+toadd:124+toadd].decode(self.string_encoding))
         name = name.replace('\x00', '')
         zdim_sizes = []
         dim_sizes = []
@@ -1517,10 +1546,7 @@ class CDF:
             return_dict['pad'] = pad
         return_dict['compression_bool'] = compression_bool
         if (compression_bool):
-            if (self.cdfversion == 3):
-                ctype, cparm = self._read_cpr(CPRorSPRoffset)
-            else:
-                ctype, cparm = self._read_cpr2(CPRorSPRoffset)
+            ctype, cparm = self._read_cpr(CPRorSPRoffset)
             return_dict['compression_level'] = cparm
         else:
             return_dict['compression_level'] = 0
@@ -1532,11 +1558,17 @@ class CDF:
         return return_dict
 
     def _read_vdr_fast(self, byte_loc):
+        if self.cdfversion == 3:
+            return self._read_vdr_fast3(byte_loc)
+        else:
+            return self._read_vdr_fast2(byte_loc)
+
+    def _read_vdr_fast3(self, byte_loc):
         with self.file.open('rb') as f:
             f.seek(byte_loc+12, 0)
             next_vdr = int.from_bytes(f.read(8), 'big', signed=True)
             f.seek(byte_loc+84, 0)
-            name = str(f.read(256).decode('utf-8'))
+            name = str(f.read(256).decode(self.string_encoding))
 
         name = name.replace('\x00', '')
 
@@ -1552,7 +1584,7 @@ class CDF:
             f.seek(byte_loc+8, 0)
             next_vdr = int.from_bytes(f.read(4), 'big', signed=True)
             f.seek(byte_loc+toadd+64, 0)
-            name = str(f.read(64).decode('utf-8'))
+            name = str(f.read(64).decode(self.string_encoding))
 
         name = name.replace('\x00', '')
 
@@ -1857,16 +1889,16 @@ class CDF:
             dt_string += ')'
         if data_type == 52 or data_type == 51:
             # string
-            if dimensions == None:
+            if dimensions is None:
                 byte_data = bytearray(byte_stream[0:num_recs*num_elems])
                 # In each record, check for the first '\x00' (null character).
                 # If found, make all the characters after it null as well.
                 for x in range(0, num_recs):
                     y = x * num_elems
                     z = byte_data[y:y+num_elems].find(b'\x00')
-                    if (z > -1 and z < (num_elems-1)):
-                        byte_data[y+z+1:y+num_elems] = b'\x00' * (num_elems - z - 1)
-                ret = byte_data[0:num_recs*num_elems].decode('utf-8').replace('\x00', '')
+                    if (z > -1 and z < (num_elems - 1)):
+                        byte_data[y + z + 1:y + num_elems] = b'\x00' * (num_elems - z - 1)
+                ret = byte_data[0:num_recs*num_elems].decode(self.string_encoding).replace('\x00', '')
             else:
                 # Count total number of strings
                 count = 1
@@ -1875,7 +1907,7 @@ class CDF:
                 strings = []
                 if (len(dimensions) == 0):
                     for i in range(0, num_recs*count*num_elems, num_elems):
-                        string1 = byte_stream[i:i+num_elems].decode('utf-8').\
+                        string1 = byte_stream[i:i+num_elems].decode(self.string_encoding).\
                             replace('\x00', '')
                         strings.append(string1)
                 else:
@@ -1883,7 +1915,7 @@ class CDF:
                         onerec = []
                         for i in range(x*count*num_elems, (x+1)*count*num_elems,
                                        num_elems):
-                            string1 = byte_stream[i:i+num_elems].decode('utf-8')\
+                            string1 = byte_stream[i:i+num_elems].decode(self.string_encoding)\
                                 .replace('\x00', '')
                             onerec.append(string1)
                         strings.append(onerec)
@@ -1955,15 +1987,9 @@ class CDF:
     def _get_attdata(self, adr_info, entry_num, num_entry, first_entry, to_np=True):
         position = first_entry
         for _ in range(0, num_entry):
-            if (self.cdfversion == 3):
-                got_entry_num, next_aedr = self._read_aedr_fast(position)
-            else:
-                got_entry_num, next_aedr = self._read_aedr_fast2(position)
+            got_entry_num, next_aedr = self._read_aedr_fast(position)
             if entry_num == got_entry_num:
-                if (self.cdfversion == 3):
-                    aedr_info = self._read_aedr(position, to_np=to_np)
-                else:
-                    aedr_info = self._read_aedr2(position, to_np=to_np)
+                aedr_info = self._read_aedr(position, to_np=to_np)
                 return_dict = {}
                 return_dict['Item_Size'] = CDF._type_size(aedr_info['data_type'],
                                                           aedr_info['num_elements'])
@@ -2182,7 +2208,7 @@ class CDF:
             if (data == ''):
                 return ('\x00'*num_elems).encode()
             else:
-                return data.ljust(num_elems, '\x00').encode('utf-8')
+                return data.ljust(num_elems, '\x00').encode(self.string_encoding)
         elif (data_type == 32):
             data_stream = data.real.tobytes()
             data_stream += data.imag.tobytes()
@@ -2250,7 +2276,7 @@ class CDF:
         '''
 
         if (data_type == 51 or data_type == 52):
-            return [data[i:i+num_elems].decode('utf-8') for i in
+            return [data[i:i+num_elems].decode(self.string_encoding) for i in
                     range(0, num_recs*num_values*num_elems, num_elems)]
         else:
             tofrom = self._convert_option()
