@@ -7,7 +7,7 @@ Importing cdflib also imports the module CDFepoch, which handles
 CDF-based epochs.
 
 The following functions can be used to convert back and forth between
-ifferent ways to display the date.
+different ways to display the date.
 
 You can call these functions like so::
 
@@ -143,24 +143,37 @@ class CDFepoch:
 
     @staticmethod
     def breakdown(epochs, to_np: bool = False):
+        if isinstance(epochs, (list, tuple, np.ndarray)):
+            item = epochs[0]
+        else:
+            item = epochs
 
-        # Returns either a single array, or a array of arrays depending on the input
-
-        if isinstance(epochs, (int, np.int64)):
+        if isinstance(item, (int, np.int64)):
             return CDFepoch.breakdown_tt2000(epochs, to_np)
-        elif isinstance(epochs, (float, np.float64)):
+        elif isinstance(item, (float, np.float64)):
             return CDFepoch.breakdown_epoch(epochs, to_np)
-        elif isinstance(epochs, (complex, np.complex128)):
+        elif isinstance(item, (complex, np.complex128)):
             return CDFepoch.breakdown_epoch16(epochs, to_np)
-        elif isinstance(epochs, (list, tuple, np.ndarray)):
-            if isinstance(epochs[0], (int, np.int64)):
-                return CDFepoch.breakdown_tt2000(epochs, to_np)
-            elif isinstance(epochs[0], (float, np.float64)):
-                return CDFepoch.breakdown_epoch(epochs, to_np)
-            elif isinstance(epochs[0], (complex, np.complex128)):
-                return CDFepoch.breakdown_epoch16(epochs, to_np)
+        else:
+            raise TypeError('Not sure how to handle type {}'.format(type(epochs)))
 
-        raise TypeError('Not sure how to handle type {}'.format(type(epochs)))
+    @staticmethod
+    def _compose_date(years, months, days,
+                      hours=None, minutes=None, seconds=None,
+                      milliseconds=None, microseconds=None, nanoseconds=None,
+                      *extras):
+        """
+        Take date components and return a numpy datetime array.
+        """
+        years = np.asarray(years) - 1970
+        months = np.asarray(months) - 1
+        days = np.asarray(days) - 1
+        types = ('<M8[Y]', '<m8[M]', '<m8[D]', '<m8[h]',
+                 '<m8[m]', '<m8[s]', '<m8[ms]', '<m8[us]', '<m8[ns]')
+        vals = (years, months, days, hours, minutes, seconds,
+                milliseconds, microseconds, nanoseconds)
+        return sum(np.asarray(v, dtype=t) for t, v in zip(types, vals)
+                   if v is not None)
 
     @classmethod
     def to_datetime(cls, cdf_time: Union[int, Sequence[int]],
@@ -173,20 +186,10 @@ class CDFepoch:
 
         If to_np is True, then the values will be returned in a numpy array.
         """
-        time_list = cls.breakdown(cdf_time, to_np=False)
-        if isinstance(time_list[0], (float, int, complex)):  # single time
-            time_list = [time_list]
-
-        if len(time_list[0]) >= 8:
-            dt = [datetime.datetime(t[0], t[1], t[2], t[3], t[4], t[5], microsecond=t[6]*1000+t[7]) for t in time_list]
-        elif len(time_list[0]) == 7:
-            dt = [datetime.datetime(t[0], t[1], t[2], t[3], t[4], t[5], microsecond=t[6]*1000) for t in time_list]
-        elif len(time_list[0]) == 6:
-            dt = [datetime.datetime(t[0], t[1], t[2], t[3], t[4], t[5]) for t in time_list]
-        else:
-            raise ValueError('unknown cdf_time format')
-
-        return np.array(dt) if to_np else dt
+        times = cls.breakdown(cdf_time, to_np=True)
+        times = np.atleast_2d(times)
+        times = cls._compose_date(*times.T)
+        return times if to_np else times.astype('datetime64[us]').tolist()
 
     @staticmethod
     def unixtime(cdf_time, to_np: bool = False):  # @NoSelf
@@ -208,18 +211,19 @@ class CDFepoch:
             time_list = [time_list]
 
         unixtime = []
+        utc = datetime.timezone(datetime.timedelta())
         for t in time_list:
             date = ['year', 'month', 'day', 'hour', 'minute', 'second', 'microsecond']
             for i in range(0, len(t)):
                 if i > 7:
                     continue
                 elif i == 6:
-                    date[i] = 1000*t[i]
+                    date[i] = 1000 * t[i]
                 elif i == 7:
-                    date[i-1] += t[i]
+                    date[i - 1] += t[i]
                 else:
                     date[i] = t[i]
-            unixtime.append(datetime.datetime(*date).timestamp())
+            unixtime.append(datetime.datetime(*date, tzinfo=utc).timestamp())
         return np.array(unixtime) if to_np else unixtime
 
     @staticmethod
@@ -265,7 +269,7 @@ class CDFepoch:
         elif isinstance(datetimes[0], (list, tuple, np.ndarray)):
             items = len(datetimes[0])
         else:
-            raise TypeError('Not sure how to handle type {}'.format(type(datetimes)))
+            raise TypeError('Not sure how to handle type {}'.format(type(datetimes[0])))
 
         if (items == 7):
             return CDFepoch.compute_epoch(datetimes, to_np)
@@ -456,11 +460,11 @@ class CDFepoch:
                 t2 = t2s[x]
                 t3 = new_tt2000[x]
                 nansec = nansecs[x]
-                xdate = xdates[:, x].tolist()
 
-                xdate.append(0)
-                xdate.append(0)
-                xdate.append(nansec)
+                xdate = np.zeros(9)
+                xdate[:6] = xdates[:, x]
+                xdate[8] = nansec
+
                 tmpNanosecs = CDFepoch.compute_tt2000(xdate)
                 if (tmpNanosecs != t3):
                     dat0 = CDFepoch._LeapSecondsfromYMD(xdate[0],
@@ -472,32 +476,30 @@ class CDFepoch:
                     nansec = CDFepoch.SECinNanoSecs + nansec
                     tmpy = tmpy - 1
                     epoch = tmpy + CDFepoch.J2000Since0AD12hSec
-                    xdate = CDFepoch._EPOCHbreakdownTT2000(epoch)[:, 0].tolist()
-                    xdate.append(0)
-                    xdate.append(0)
-                    xdate.append(nansec)
+                    xdate = np.zeros(9)
+                    xdate[:6] = CDFepoch._EPOCHbreakdownTT2000(epoch)[:, 0]
+                    xdate[8] = nansec
                     tmpNanosecs = CDFepoch.compute_tt2000(xdate)
                 if (tmpNanosecs != t3):
                     dat0 = CDFepoch._LeapSecondsfromYMD(xdate[0],
                                                         xdate[1], xdate[2])
                     tmpx = t2 - int(dat0 * CDFepoch.SECinNanoSecs)
-                    tmpy = int((1.0*tmpx)/CDFepoch.SECinNanoSecsD)
+                    tmpy = int((1.0 * tmpx) / CDFepoch.SECinNanoSecsD)
                     nansec = int(tmpx - tmpy * CDFepoch.SECinNanoSecs)
                     if (nansec < 0):
                         nansec = CDFepoch.SECinNanoSecs + nansec
                         tmpy = tmpy - 1
                     epoch = tmpy + CDFepoch.J2000Since0AD12hSec
-                    xdate = CDFepoch._EPOCHbreakdownTT2000(epoch)[:, 0].tolist()
-                    xdate.append(0)
-                    xdate.append(0)
-                    xdate.append(nansec)
+                    xdate = np.zeros(9)
+                    xdate[:6] = CDFepoch._EPOCHbreakdownTT2000(epoch)[:, 0]
+                    xdate[8] = nansec
                     tmpNanosecs = CDFepoch.compute_tt2000(xdate)
                     if (tmpNanosecs != t3):
                         dat0 = CDFepoch._LeapSecondsfromYMD(xdate[0],
                                                             xdate[1],
                                                             xdate[2])
                         tmpx = t2 - int(dat0 * CDFepoch.SECinNanoSecs)
-                        tmpy = int((1.0*tmpx)/CDFepoch.SECinNanoSecsD)
+                        tmpy = int((1.0 * tmpx) / CDFepoch.SECinNanoSecsD)
                         nansec = int(tmpx - tmpy * CDFepoch.SECinNanoSecs)
                         if (nansec < 0):
                             nansec = CDFepoch.SECinNanoSecs + nansec
@@ -532,7 +534,7 @@ class CDFepoch:
     @staticmethod
     def compute_tt2000(datetimes, to_np: bool = False):
 
-        if not isinstance(datetimes, (list, tuple)):
+        if not isinstance(datetimes, (list, tuple, np.ndarray)):
             raise TypeError('datetime must be in list form')
 
         if isinstance(datetimes[0], numbers.Number):
@@ -1417,7 +1419,7 @@ class CDFepoch:
                 if not to_np:
                     return (86400000.0*daysSince0AD+msecInDay)
                 else:
-                    return np.array((86400000.0*daysSince0AD+msecInDay))
+                    return np.array(86400000.0*daysSince0AD+msecInDay)
             epochs.append(86400000.0*daysSince0AD+msecInDay)
         if not to_np:
             return epochs
